//FRAGMENT SHADER

// inputs
uniform lowp vec4 blendColor;
uniform lowp vec4 noBlackBlendColor;

uniform highp vec4 pixelToPhoto;

uniform lowp float blurIntensity;
uniform lowp float additiveIntensity;
uniform lowp float contrastIntensity;
uniform lowp float brightnessIntensity;
uniform lowp float differenceIntensity;
uniform lowp float saturationIntensity;

// general variables for laying out blend
uniform sampler2D image_texture; // the image texture object

varying highp vec2 v_baseTextureCoordinate; // texture coordinates for the image
uniform sampler2D glow_texture; // the image texture object

varying highp vec2 v_textureCoordinate; // texture coordinates for the image

#ifdef USES_REDBLUE
varying highp vec2 v_redTextureCoordinate; // texture coordinates for red
#endif

#ifdef USES_BLUR
varying highp vec2 v_blurTexCoords[4];
varying lowp float bias;
#endif

//varying lowp float localBlurIntensity;
varying lowp float localContrastIntensity;
varying lowp float localBrightnessIntensity;
varying lowp float localAdditiveIntensity;
varying lowp float localDifferenceIntensity;
varying lowp float localSaturationIntensity;

// gausian filter weights creating using: http://www.embege.com/gauss/

#ifdef USES_WRAP_COMPAT
/**
 * Tegra GPUs don't support non-clamped modes for NPOT textures, so
 * we have to repeat by hand.
 */
highp vec2 mirroredRepeatCompat(highp vec2 texCoords) {
  highp float xMod2 = mod(texCoords.x, 2.0);
  if (xMod2 > 1.0) xMod2 = 2.0 - xMod2;

  highp float yMod2 = mod(texCoords.y, 2.0);
  if (yMod2 > 1.0) yMod2 = 2.0 - yMod2;

  return vec2(xMod2, yMod2);
}
#endif

void main() {
    
    mediump vec4 imageTexelColor;

    highp vec2 texCoord = v_textureCoordinate;
#ifdef USES_WRAP_COMPAT
    texCoord = mirroredRepeatCompat(v_textureCoordinate);
#endif

#ifdef USES_REDBLUE
  highp vec2 redTexCoord = v_redTextureCoordinate;
#ifdef USES_WRAP_COMPAT
  redTexCoord = mirroredRepeatCompat(v_redTextureCoordinate);
#endif
#endif
    
    // apply the blur
    
#ifdef USES_BLUR
    
    // center pixel
    imageTexelColor = texture2D(image_texture, texCoord, bias)*0.20417995557165622;
    
    
    // above and below
    highp vec2 blurCoords[4];
#ifdef USES_WRAP_COMPAT
    blurCoords[0] = mirroredRepeatCompat(v_blurTexCoords[0]);
    blurCoords[1] = mirroredRepeatCompat(v_blurTexCoords[1]);
    blurCoords[2] = mirroredRepeatCompat(v_blurTexCoords[2]);
    blurCoords[3] = mirroredRepeatCompat(v_blurTexCoords[3]);
#else
    blurCoords[0] = v_blurTexCoords[0];
    blurCoords[1] = v_blurTexCoords[1];
    blurCoords[2] = v_blurTexCoords[2];
    blurCoords[3] = v_blurTexCoords[3];
#endif

    imageTexelColor += texture2D(image_texture, blurCoords[ 0], bias)*0.1989550111;
    imageTexelColor += texture2D(image_texture, blurCoords[ 1], bias)*0.1989550111;
    imageTexelColor += texture2D(image_texture, blurCoords[ 2], bias)*0.1989550111;
    imageTexelColor += texture2D(image_texture, blurCoords[ 3], bias)*0.1989550111;
    
    #ifdef USES_REDBLUE
        
        // use the bias if we're blurring to give it a little blur
        imageTexelColor.r = texture2D(image_texture, redTexCoord, bias).r;
        
    #endif
    
    
#else
    
    imageTexelColor =  texture2D(image_texture, texCoord);
    
    #ifdef USES_REDBLUE
        
        imageTexelColor.r = texture2D(image_texture, redTexCoord).r;
        
    #endif
    
#endif

    
#ifdef USES_DIFFERENCE
    // apply the difference
    imageTexelColor = abs(imageTexelColor - (noBlackBlendColor * localDifferenceIntensity));
#endif

#ifdef USES_BRIGHTNESS
    // apply the brightness
    imageTexelColor = imageTexelColor + (localBrightnessIntensity * noBlackBlendColor);

#endif
   
#ifdef USES_CONTRAST
    // apply the contrast
    imageTexelColor = (imageTexelColor - 0.5) * (1.0 + (localContrastIntensity * 3.0)) + 0.5;
    imageTexelColor = imageTexelColor * (1.0 - localContrastIntensity) + (imageTexelColor * blendColor) * localContrastIntensity;

#endif
    
#ifdef USES_ADDITIVE
    // apply the additive blend
    highp vec4 blendedTexelColor = imageTexelColor + (imageTexelColor * localAdditiveIntensity * 2.0 * noBlackBlendColor);
    imageTexelColor = (imageTexelColor * (1.0 - localAdditiveIntensity)) + (blendedTexelColor * localAdditiveIntensity);
#endif
    
#ifdef USES_SATURATION
    // apply the saturation blend
    highp vec3 grayXfer = vec3(0.3, 0.59, 0.11);
	highp vec4 gray = vec4(dot(grayXfer, imageTexelColor.rgb));
    imageTexelColor = (imageTexelColor * (1.0 - localSaturationIntensity)) + (gray * localSaturationIntensity);
#endif
    
    imageTexelColor.a = 1.0;
    

    
// don't draw glow if we're drawing photo only
#if defined(USES_GLOW) && !defined(IS_DRAWING_PHOTO_ONLY)

    lowp vec4 glowColor = texture2D(glow_texture, v_baseTextureCoordinate);

    #ifdef INVERT_GLOW
        imageTexelColor.rgb = mix(imageTexelColor.rgb, glowColor.rgb, 1.0-glowColor.a);
    #else
          imageTexelColor.rgb = mix(imageTexelColor.rgb, glowColor.rgb, glowColor.a);
    #endif
        

    // if we're drawing the fragment only then just apply glow to a fully transparent color
    #if defined(IS_PHOTO_PASS) && defined(IS_DRAWING_TOP_LAYER_ONLY)
        imageTexelColor = glowColor;
        imageTexelColor.rgb = imageTexelColor.rgb * imageTexelColor.a;
    #endif
    
#endif
    
    gl_FragColor = imageTexelColor;
}



